/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file GeneralSettingsFileManager.h
   
  \brief Class responsible for managing values added or removed to the file referenced by QSettings.
    QSettings (Doc): Users normally expect an application to remember its settings (window sizes and positions, options, etc.) across sessions. 
    This information is often stored in the system registry on Windows, and in XML preferences files on Mac OS X. 
    On Unix systems, in the absence of a standard, many applications (including the KDE applications) use INI text files.
    Scope: user.
    Default settings: "TerraLib"(Organization) and "TerraPrint"(Application)

  \ingroup layout
*/

#ifndef __TERRALIB_LAYOUT_INTERNAL_GENERAL_SETTINGS_FILE_MANAGER_H
#define __TERRALIB_LAYOUT_INTERNAL_GENERAL_SETTINGS_FILE_MANAGER_H

// TerraLib
#include "../../core/Config.h"

#include <QString>
#include <QMap>
#include <QMap>

class QVariant;

namespace te
{
  namespace layout
  {
    class SharedIOAccess;
    /*!
      \brief Class responsible for managing values added or removed to the file referenced by QSettings.
      QSettings (Doc): Users normally expect an application to remember its settings (window sizes and positions, options, etc.) across sessions. 
      This information is often stored in the system registry on Windows, and in XML preferences files on Mac OS X. 
      On Unix systems, in the absence of a standard, many applications (including the KDE applications) use INI text files.
      Scope: user.
      Default settings: "TerraLib"(Organization) and "TerraPrint"(Application)
    
    \ingroup layout
    */
    class TELAYOUTEXPORT GeneralSettingsFileManager
    {
      public:

        /*!
          \brief Constructor
        */
        GeneralSettingsFileManager(SharedIOAccess* sharedIOAccess, const QString& organization = "", const QString& application = "");

        /*!
          \brief Destructor
        */
        virtual ~GeneralSettingsFileManager();

        QVariant getSetting(const QString& key);

        bool writeSetting(const QString& key, const QVariant& variant);

        bool writeSetting(const QString& group, const QString& key, const QVariant& variant);

        bool writeSettings(const QString& group, const QMap<QString, QVariant>& listSettings);

        QMap<QString, QVariant> getGroupSettings(const QString& group);
        
        bool removeSetting(const QString& key);

        bool removeSetting(const QString& group, const QString& key);

        QString getDefaultImageGalleryGroup();

        /*!
        \brief Default TerraPrint prefix.
        */
        QString getDefaultLayoutAlias();

        /*!
          \brief Default TerraPrint prefix.
        */
        QString getDefaultLayoutPrefix();

        /*!
          \brief Add a pair key/value to a internal list of prefixes. 
            The prefix has to follow the file path form of the Qt Resource System ":/".

          \ param alias will be the key
          \ param prefix will be the value
        */
        bool addPrefix(const QString& alias, const QString& prefix);

        /*!
        \brief Remove a pair key/value of a internal list of prefixes.

        \ param alias the key
        */
        bool removePrefix(const QString& alias);

        /*!
          \brief Get all the registered prefixes. Pair key (alias) and value (prefix).
        */
        QMap<QString, QString> getPrefixList();

      protected:

        SharedIOAccess*            m_sharedIOAccess;
        QString                    m_organization;
        QString                    m_application;
        QString                    m_defaultImageGalleryGroup;
        QMap<QString, QString>     m_prefixList;
        QString                    m_defaultLayoutAlias;
        QString                    m_defaultLayoutPrefix;
    };
  }
}

#endif 
