/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/statistics/core/Utils.h

  \brief Utility functions for Statistics.
*/

#ifndef __TERRALIB_ATTRIBUTEFILL_CORE_INTERNAL_UTILS_H
#define __TERRALIB_ATTRIBUTEFILL_CORE_INTERNAL_UTILS_H

// Terralib
#include "Config.h"
#include "Enums.h"

#include "../geometry/Coord2D.h"
#include "../geometry/Geometry.h"
#include "../geometry/Polygon.h"
#include "../raster/Grid.h"
#include "../raster/Raster.h"

// STL
#include <map>
#include <string>

namespace te
{
  namespace attributefill
  {
    /*!
      \brief Gets the full name of the operation as string.

      \param e   The enumerator.

      \return The full name for the operation.
    */
    TEATTRIBUTEFILLEXPORT std::string GetOperationFullName(const int& e);

    /*!
      \brief Convert the coordinate to the grid col/row postion of the pixel .

      If the coordinate is outside the grid extent the function will
      return std::numeric_limits<uint32_t>::max for col and/or row the is outside.
      The other value is valid.

      \return Pair of <col, row> of the pixel at the coord.
    */
    TEATTRIBUTEFILLEXPORT std::pair<uint32_t, uint32_t> GeoToGrid(const te::gm::Coord2D& coord,
                                                                  const rst::Grid &grid);

    /*!
      \brief Gets the minimum and maximum row and column values of the raster based on polygon.

      \param raster         The raster.
      \param polygon        The polygon.
      \param minimumRow     Minimum value for row
      \param minimumColumn  Minimum value for column
      \param maximumRow     Maximum value for row
      \param maximumColumn  Maximum value for column

      \return Fills the minimum and maximum values.
    */
    TEATTRIBUTEFILLEXPORT void GetMinMaxLineAndColumn(const te::rst::Raster &raster,
                                                      const te::gm::Polygon &polygon,
                                                      uint32_t &minimumRow, uint32_t &minimumColumn,
                                                      uint32_t &maximumRow, uint32_t &maximumColumn);

    /*!
      \brief Gets the pixel values for a specific band that intersects the polygon.

      The computation is based on the intersection of the pixel resolution and the polygon.

      \param raster         The raster.
      \param band           The band.
      \param polygon        The polygon.
      \param minimumRow     Minimum value for row
      \param minimumColumn  Minimum value for column
      \param maximumRow     Maximum value for row
      \param maximumColumn  Maximum value for column
      \param values         A map<key, value> where the key will be filled with the pixel and the value with the number of occurrence.

      \return A map<key, value> of values that the key is the pixel and the value is the number of occurrence.
    */
    TEATTRIBUTEFILLEXPORT void GetValuesFromBand(const te::rst::Raster& raster,
                                                 const unsigned int &band,
                                                 const gm::Polygon &polygon,
                                                 const uint32_t &minimumRow,
                                                 const uint32_t &minimumColumn,
                                                 const uint32_t &maximumRow,
                                                 const uint32_t &maximumColumn,
                                                 std::map<double, int>& values);

    /*!
      \brief Gets the pixel percentage for a specific band that intersects the polygon.

      The computation is based on the intersection of the pixel resolution and the polygon.

      \param raster                     The raster.
      \param band                       The band.
      \param polygon                    The polygon.
      \param minimumRow                 Minimum value for row
      \param minimumColumn              Minimum value for column
      \param maximumRow                 Maximum value for row
      \param maximumColumn              Maximum value for column
      \param percentOfEachClassByArea   A map<key, value> where the key will be filled with the pixel and the value with the percentage of occurence.

      \return A map<key, value> of values that the key is the pixel and the value is the percentage of occurrence.
    */
    TEATTRIBUTEFILLEXPORT void GetPercentOfEachClassByArea(const te::rst::Raster &raster,
                                                           const unsigned int &band,
                                                           const gm::Polygon &polygon,
                                                           const uint32_t &minimumRow,
                                                           const uint32_t &minimumColumn,
                                                           const uint32_t &maximumRow,
                                                           const uint32_t &maximumColumn,
                                                           std::map<double, double> &percentOfEachClassByArea);

  } // end namespace attributefill
}   // end namespace te

#endif  // __TERRALIB_ATTRIBUTEFILL_CORE_INTERNAL_UTILS_H
