/*  Copyright (C) 2001-2009 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/edit/qt/tools/DeleteGeometryByAreaTool.h

  \brief This class implements a concrete tool to remove geometries by drawing an area (polygon).
*/

#ifndef __TERRALIB_EDIT_QT_INTERNAL_DELETEGEOMETRYBYAREATOOL_H
#define __TERRALIB_EDIT_QT_INTERNAL_DELETEGEOMETRYBYAREATOOL_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/dataset/ObjectId.h"
#include "../../../geometry/Envelope.h"
#include "../../../geometry/Geometry.h"
#include "../../../geometry/Utils.h"
#include "../../../maptools/AbstractLayer.h"
#include "../../Utils.h"
#include "GeometriesUpdateTool.h"
#endif
#include "../Config.h"

// Qt
#include <QPointF>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class Canvas;
      class MapDisplay;
    }
  }

  namespace edit
  {
// Forward declaration
    class Feature;

    /*!
      \class DeleteGeometryByTool

      \brief This class implements a concrete tool to remove geometries by drawing an area (polygon).
    */
    class TEEDITQTEXPORT DeleteGeometryByAreaTool : public GeometriesUpdateTool
    {
      Q_OBJECT

      public:

        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
        //@{

        /*!
          \brief It constructs a remove geometries tool associated with the given map display.

          \param display The map display associated with the tool.
          \param parent The tool's parent.

          \note The tool will NOT take the ownership of the given pointers.
        */
        DeleteGeometryByAreaTool(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, QObject* parent = 0);

        /*! \brief Destructor. */
        ~DeleteGeometryByAreaTool();

        //@}

        /** @name AbstractTool Methods
          *  Methods related with tool behavior.
          */
        //@{

        bool mousePressEvent(QMouseEvent* e);

        bool mouseMoveEvent(QMouseEvent* e);

        bool mouseReleaseEvent(QMouseEvent* e);

        //@}

      private:

        void draw();

        void drawPolygon();

        te::gm::Geometry* buildPolygon();

        te::gm::Envelope buildEnvelope(const QPointF& pos);

        void executeSelection(const te::gm::Envelope& e);

        void storeFeature();

        void updateCursor();

      protected:

      std::vector<te::gm::Coord2D> m_coords;  //!< The coord list managed by this tool.
      te::gm::Coord2D m_lastPos;              //!< The last position captured on mouse move event.
      bool m_selectionStarted;                //!< Flag that indicates if selection operation was started.
      bool m_keepPreviousSelection;           //!< Flag that indicates if the tool must keep the previous selection.
      bool m_selectionByPointing;             //!< Flag that indicates if the selection is by pointing.
      te::gm::Geometry* m_area;

    };
  }   // end namespace edit
}     // end namespace te

#endif  // __TERRALIB_EDIT_QT_INTERNAL_DELETEGEOMETRYBYAREATOOL_H
