/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/RpToolsWidget.h

  \brief This file has the RpToolsWidget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_ROTOOLSWIDGET_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_ROTOOLSWIDGET_H

// TerraLib
#include "../../../geometry/Envelope.h"
#ifndef Q_MOC_RUN
#include "../../../geometry/Polygon.h"
#include "../../../maptools/AbstractLayer.h"
#endif
#include "../../../maptools/Enums.h"
#include "../Config.h"

// STL
#include <memory>

// Qt
#include <QComboBox>
#include <QWidget>
#include <qactiongroup.h>

namespace Ui { class RpToolsWidgetForm; }

namespace te
{
  namespace gm  { class Envelope; }
  namespace rst { class Raster; }
  namespace qt
  {
    namespace widgets
    {
      class AbstractTool;
      class MapDisplay;

      /*!
        \class RpToolsWidget

        \brief This class is used to navigate over a DataSetLayer (having a raster representation)
           and given a set of tools, such as, zoom in, zoom out, pan, recompose.
        Two new tools as created for raster interaction:
        - point clicked
        - geom definition

        \sa RasterFactory
      */
      class TEQTWIDGETSEXPORT RpToolsWidget : public QWidget
      {
        Q_OBJECT

        public:

          RpToolsWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~RpToolsWidget();

        public:

          /*!
            \brief This method is used to set the selected layer
            
            \param layer The layer ptr 

            \note This layer MUST HAVE a valid raster object.
          */
          void set(te::map::AbstractLayerPtr layer);

          void setVectorial(te::map::AbstractLayerPtr layer);

          void removeVectorial();

          void setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay);

          void setActionGroup(QActionGroup* actionGroup);

          void hideEditionTools(bool hide);

          void enablePickerAction();

          void enableGeomAction();

          void enableBoxAction();

          void setPickerTool(bool checked);

          void setGeomTool(bool checked);

          void setEnvelopeTool(bool checked);

          void setSelectionMode(bool mode);

          void onDisableToolbar();

        protected slots:

          void onEnvelopeAcquired(te::gm::Envelope env);

          void onGeomAquired(te::gm::Polygon* poly);

          void onPointPicked(QPointF& point);

          void onMapDisplayExtentChanged();

          void onPointPickerToggled(bool checked);
          
          void onGeomToggled(bool checked);

          void onBoxToggled(bool checked);

        signals:

          void mapDisplayExtentChanged();

          void pointPicked(double x, double y);

          void geomAquired(te::gm::Polygon* poly);

          void envelopeAcquired(te::gm::Envelope env);

        private:

          std::auto_ptr<Ui::RpToolsWidgetForm> m_ui;

          te::map::AbstractLayerPtr m_layer;

          te::qt::widgets::MapDisplay* m_mapDisplay;

          QActionGroup* m_actionGroup;

          QAction* m_pickerAction;

          QAction* m_geomAction;

          QAction* m_boxAction;

          QCursor m_pointCursor;

          QPixmap* m_draftOriginal;
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_RP_INTERNAL_ROTOOLSWIDGET_H

